/*
** pause.c -- timing functions
**
** Copyright 1992, 1993, 1994 by
** Michael J. Gourlay
**
** NO WARRANTEES, EXPRESS OR IMPLIED.
*/

#include "showtime.h"
#include "pause.h"

#ifdef X11
#include <sys/types.h>
#include <time.h>
#endif


/* ====================================================================== */
/* ---------------------------------------------------------------------- */
/* ====================================================================== */

static long gm_begin_timeval;

/* pause_init -- start timer used with pause_finish
*/
void
pause_init()
{
#if defined(MSC) || defined(TC)
  /* For IBMPC's, the BIOS clock ticks 18.2 times per second */
  /* Get the initial clock time */
  TICK_TOCK(gm_begin_timeval);
#endif
}

/* ---------------------------------------------------------------------- */

/* pause_finish -- wait for the specified number of seconds after pause_init
** If seconds is negative, allow keypress to resume
*/
void
pause_finish(float seconds)
{
#if defined(MSC) || defined(TC)
  long end_timeval;
  register char step=0;

  /* For IBMPC's, the BIOS clock ticks 18.2 times per second */

  if(seconds==0.0) return;

  /* Set the step flag and correct for negative time if necessary */
  if(seconds<0.0) { step=1; seconds=-seconds; }

  while(!(step && (kbhit() && (getch()||1))))  {
    TICK_TOCK(end_timeval);
    if(end_timeval<gm_begin_timeval) end_timeval += TICK_WRAP;
    if((long)(seconds*TICK_TIME) <= (end_timeval-gm_begin_timeval)) return;
  }
#endif
}

/* ---------------------------------------------------------------------- */

/* pause -- wait for the specified number of seconds
** If seconds is negative, allow keypress to resume
*/
void
st_pause(float seconds)
{
  register char step=0;

  /* For IBMPC's, the BIOS clock ticks 18.2 times per second */
#if defined(MSC) || defined(TC)
  long begin_timeval, end_timeval;
#elif defined(X11)
  time_t begin_timeval, end_timeval;
#endif

  /* Get the initial clock time */
  TICK_TOCK(begin_timeval);

  /* Set the step flag and correct for negative time if necessary */
  if(seconds<0.0) { step=1; seconds=-seconds; }

#if defined(MSC) || defined(TC)
  while(!(step && (KBHIT() && (getch()||1))))  {
    TICK_TOCK(end_timeval);
    if(end_timeval<begin_timeval) end_timeval += TICK_WRAP;
    if((seconds*TICK_TIME) <= DIFFTIME(end_timeval, begin_timeval))
      return;
    DELAY(1);
  }
#endif
}
